@@ -26,6 +26,7 @@ between the MIB and SNMP.
 import struct
 import socket
 from datetime import timedelta
+from pysnmp.proto import rfc1902
 
 import mib, snmp
 
@@ -64,10 +65,10 @@ class Type(object):
     @classmethod
     def _internal(cls, entity, value):
         """Get internal value for a given value."""
-        raise NotImplementedError
+        raise NotImplementedError # pragma: no cover
 
     def pack(self):
-        raise NotImplementedError
+        raise NotImplementedError # pragma: no cover
 
     def toOid(self):
         """Convert to an OID.
@@ -80,7 +81,7 @@ class Type(object):
 
         @return: OID that can be used as index
         """
-        raise NotImplementedError
+        raise NotImplementedError # pragma: no cover
 
     @classmethod
     def fromOid(cls, entity, oid):
@@ -94,7 +95,7 @@ class Type(object):
            needed to create the instance and v the instance created from
            the OID
         """
-        raise NotImplementedError
+        raise NotImplementedError # pragma: no cover
 
     @classmethod
     def _fixedOrImplied(cls, entity):
@@ -108,7 +109,7 @@ class Type(object):
         @return: C{fixed} if it is fixed-len, C{implied} if implied var-len,
            C{False} otherwise
         """
-        if entity.ranges and type(entity.ranges) is not tuple:
+        if entity.ranges and type(entity.ranges) is not tuple and type(entity.ranges) is not list:
             # Fixed length
             return "fixed"
 
@@ -146,14 +147,16 @@ class IpAddress(Type):
         if isinstance(value, list) or isinstance(value, tuple):
             value = ".".join([str(a) for a in value])
         try:
-            value = socket.inet_ntoa(socket.inet_aton(value))
+            value = socket.inet_ntoa(value)
         except:
-            raise ValueError("%r is not a valid IP" % value)
+            try:
+                value = socket.inet_ntoa(socket.inet_aton(value))
+            except:
+                raise ValueError("%r is not a valid IP" % value)
         return [int(a) for a in value.split(".")]
 
     def pack(self):
-        return (snmp.ASN_IPADDRESS,
-                socket.inet_aton(".".join(["%d" % x for x in self._value])))
+        return rfc1902.IpAddress(".".join(["%d" % x for x in self._value]))
 
     def toOid(self):
         return tuple(self._value)
@@ -172,7 +175,7 @@ class IpAddress(Type):
             try:
                 other = IpAddress(self.entity, other)
             except:
-                raise NotImplementedError
+                raise NotImplementedError # pragma: no cover
         if self._value == other._value:
             return 0
         if self._value < other._value:
@@ -190,7 +193,7 @@ class String(Type, str):
         return str(value)
 
     def pack(self):
-        return (snmp.ASN_OCTET_STR, self._value)
+        return rfc1902.OctetString(self._value)
 
     def toOid(self):
         # To convert properly to OID, we need to know if it is a
@@ -353,15 +356,11 @@ class Integer(Type, long):
         if self._value >= (1L << 64):
             raise OverflowError("too large to be packed")
         if self._value >= (1L << 32):
-            # Pack in a 64 bit counter
-            return (snmp.ASN_OCTET_STR,
-                    struct.pack("LL",
-                                self._value/(1L << 32),
-                                self._value%(1L << 32)))
+            return rfc1902.Counter64(self._value)
         if self._value >= 0:
-            return (snmp.ASN_INTEGER, struct.pack("L", self._value))
+            return rfc1902.Integer(self._value)
         if self._value >= -(1L << 31):
-            return (snmp.ASN_INTEGER, struct.pack("l", self._value))
+            return rfc1902.Integer(self._value)
         raise OverflowError("too small to be packed")
 
     def toOid(self):
@@ -405,7 +404,7 @@ class Unsigned32(Integer):
             raise OverflowError("too large to be packed")
         if self._value < 0:
             raise OverflowError("too small to be packed")
-        return (snmp.ASN_UNSIGNED, struct.pack("L", self._value))
+        return rfc1902.Unsigned32(self._value)
 
 class Unsigned64(Integer):
     def pack(self):
@@ -413,8 +412,7 @@ class Unsigned64(Integer):
             raise OverflowError("too large to be packed")
         if self._value < 0:
             raise OverflowError("too small to be packed")
-        return (snmp.ASN_COUNTER64, struct.pack("LL",
-            self._value/(1L << 32), self._value%(1L << 32)))
+        return rfc1902.Counter64(self._value)
 
 class Enum(Integer):
     """Class for enumeration"""
@@ -433,7 +431,7 @@ class Enum(Integer):
                                                                  entity))
 
     def pack(self):
-        return (snmp.ASN_INTEGER, struct.pack("l", self._value))
+        return rfc1902.Integer(self._value)
 
     @classmethod
     def fromOid(cls, entity, oid):
@@ -446,7 +444,7 @@ class Enum(Integer):
             try:
                 other = self.__class__(self.entity, other)
             except:
-                raise NotImplementedError
+                raise NotImplementedError # pragma: no cover
         return self._value == other._value
 
     def __ne__(self, other):
@@ -474,9 +472,9 @@ class Oid(Type):
             return tuple(value.oid)
         else:
             raise TypeError("don't know how to convert %r to OID" % value)
-            
+
     def pack(self):
-        return (snmp.ASN_OBJECT_ID, "".join([struct.pack("l", x) for x in self._value]))
+        return rfc1902.univ.ObjectIdentifier(self._value)
 
     def toOid(self):
         if self._fixedOrImplied(self.entity):
@@ -566,18 +564,18 @@ class Timeticks(Type):
         return (1, cls(entity, oid[0]))
 
     def pack(self):
-        return (snmp.ASN_INTEGER,
-                struct.pack("l",
-                            int(self)))
+        return rfc1902.TimeTicks(int(self))
 
     def __str__(self):
         return str(self._value)
 
     def __cmp__(self, other):
-        if isinstance(other, int) or isinstance(other, long):
+        if isinstance(other, Timeticks):
+            other = other._value
+        elif isinstance(other, int) or isinstance(other, long):
             other = timedelta(0, other/100.)
         elif not isinstance(other, timedelta):
-            raise NotImplementedError("only compare to int or timedelta")
+            raise NotImplementedError("only compare to int or timedelta, not %s" % type(other))
         if self._value == other:
             return 0
         if self._value < other:
@@ -642,7 +640,7 @@ class Bits(Type):
             if len(string) < b/16 + 1:
                 string.extend([0]*(b/16 - len(string)+1))
             string[b/16] |= 1 << (7 - b%16)
-        return (snmp.ASN_OCTET_STR, "".join([chr(x) for x in string]))
+        return rfc1902.Bits("".join([chr(x) for x in string]))
 
     def __eq__(self, other):
         if isinstance(other, str):
