@@ -26,7 +26,16 @@ from time import time
 from UserDict import DictMixin
 import snmp, mib, basictypes
 
-class DelayedSetSession(object):
+class DelegatedSession(object):
+    """General class for SNMP session for delegation"""
+    def __init__(self, session):
+        self._session = session
+    def __getattr__(self, attr):
+        return getattr(self._session, attr)
+    def __setattribute__(self, attr, value):
+        return setattr(self._session, attr, value)
+
+class DelayedSetSession(DelegatedSession):
     """SNMP session that is able to delay SET requests.
 
     This is an adapter. The constructor takes the original (not
@@ -34,21 +43,17 @@ class DelayedSetSession(object):
     """
 
     def __init__(self, session):
-        self.session = session
+        DelegatedSession.__init__(self, session)
         self.setters = []
 
-    def get(self, *args):
-        return self.session.get(*args)
-    def getnext(self, *args):
-        return self.session.getnext(*args)
     def set(self, *args):
         self.setters.extend(args)
 
     def commit(self):
         if self.setters:
-            self.session.set(*self.setters)
+            self._session.set(*self.setters)
 
-class NoneSession(object):
+class NoneSession(DelegatedSession):
     """SNMP session that will return None on unsucessful GET requests.
 
     In a normal session, a GET request returning `No such instance`
@@ -56,14 +61,9 @@ class NoneSession(object):
     error and return None instead.
     """
 
-    def __init__(self, session):
-        self.session = session
-        self.set = self.session.set
-        self.getnext = self.session.getnext
-
     def get(self, *args):
         try:
-            return self.session.get(*args)
+            return self._session.get(*args)
         except (snmp.SNMPNoSuchName,
                 snmp.SNMPNoSuchObject,
                 snmp.SNMPNoSuchInstance):
@@ -73,28 +73,25 @@ class NoneSession(object):
                 raise
             return ((args[0], None),)
 
-class CachedSession(object):
+class CachedSession(DelegatedSession):
     """SNMP session using a cache.
 
     This is an adapter. The constructor takes the original session.
     """
 
     def __init__(self, session, timeout=5):
-        self.session = session
+        DelegatedSession.__init__(self, session)
         self.cache = {}
         self.timeout = timeout
         self.count = 0
 
-    def set(self, *args):
-        return self.session.set(*args)
-
     def getorgetnext(self, op, *args):
         self.count += 1
         if (op, args) in self.cache:
             t, v = self.cache[op, args]
             if time() - t < self.timeout:
                 return v
-        value = getattr(self.session, op)(*args)
+        value = getattr(self._session, op)(*args)
         self.cache[op, args] = [time(), value]
         self.flush()
         return value
@@ -120,20 +117,18 @@ class Manager(object):
 
     def __init__(self,
                  host="localhost",
-                 community=None, version=None,
+                 community=None, version=2,
                  cache=False, none=False,
                  timeout=None, retries=None,
                  # SNMPv3
-                 seclevel=None, secname=None,
+                 secname=None,
                  authprotocol=None, authpassword=None,
                  privprotocol=None, privpassword=None):
         if host is None:
             host = Manager._host
         self._host = host
-        if version is None:
-            version = -1
         self._session = snmp.Session(host, community, version,
-                                     seclevel, secname,
+                                     secname,
                                      authprotocol, authpassword,
                                      privprotocol, privpassword)
         if timeout is not None:
@@ -262,41 +257,29 @@ class ProxyColumn(Proxy, DictMixin):
         oid = self.proxy.oid
         indexes = self.proxy.table.index
 
-        while True:
-            try:
-                if self.session.bulk and self.session.use_bulk:
-                    results = self.session.getbulk(oid)
-                else:
-                    results = self.session.getnext(oid)
-            except snmp.SNMPEndOfMibView:
+        for noid, result in self.session.walk(oid):
+            if noid <= oid:
+                noid = None
+                break
+            oid = noid
+            if not((len(oid) >= len(self.proxy.oid) and
+                oid[:len(self.proxy.oid)] == self.proxy.oid[:len(self.proxy.oid)])):
+                noid = None
                 break
 
-            for noid, result in results:
-                if noid <= oid:
-                    noid = None
-                    break
-                oid = noid
-                if not((len(oid) >= len(self.proxy.oid) and
-                    oid[:len(self.proxy.oid)] == self.proxy.oid[:len(self.proxy.oid)])):
-                    noid = None
-                    break
-
-                # oid should be turned into index
-                index = oid[len(self.proxy.oid):]
-                target = []
-                for x in indexes:
-                    l, o = x.type.fromOid(x, tuple(index))
-                    target.append(x.type(x, o))
-                    index = index[l:]
-                count = count + 1
-                if len(target) == 1:
-                    # Should work most of the time
-                    yield target[0], result
-                else:
-                    yield tuple(target), result
-
-            if noid is None:
-                break
+            # oid should be turned into index
+            index = oid[len(self.proxy.oid):]
+            target = []
+            for x in indexes:
+                l, o = x.type.fromOid(x, tuple(index))
+                target.append(x.type(x, o))
+                index = index[l:]
+            count = count + 1
+            if len(target) == 1:
+                # Should work most of the time
+                yield target[0], result
+            else:
+                yield tuple(target), result
 
         if count == 0:
             # We did not find any element. Is it because the column is
